@@ -2,29 +2,33 @@ from .logging import clog
 from .config import (
     CONDA_LINUX_INSTALLER_URL,
     get_project_remote_dict_config,
-    get_project_dict_config,
     get_project_settings_for_remote,
     get_project_env,
     randomString,
-    ver_re,
     CONDA_MIN_VERSION,
+    read_project_remote_config,
+    write_project_remote_config,
 )
 from .exception import MissingDependency, RemoteCommandError, LocalCommandError
-from .shell import run_shell_cmd, run_ssh_cmd, quote
-from .remote import write_dict_to_remote_db
-from .platform import find_project_dir, is_linux_64, get_python_version
-from .exception import NotAProjectFolderError, ResolosException, DependencyVersionError
+from .shell import run_shell_cmd, run_ssh_cmd
+from .platform import find_project_dir, is_linux_64
+from .exception import ResolosException, DependencyVersionError, SSHError
 from .unison import sync_files
 import pathlib
 import json
 import ast
 from semver import VersionInfo
+from datetime import datetime
+import re
+
+
+conda_ver_re = re.compile(r"conda (\d+.\d+.\d+)")
 
 
 def verify_conda_version(output):
-    m = ver_re.search(output)
+    m = conda_ver_re.search(output)
     if m:
-        conda_ver = VersionInfo.parse(m.group())
+        conda_ver = VersionInfo.parse(m.group(1))
         if conda_ver < CONDA_MIN_VERSION:
             raise DependencyVersionError(
                 f"Resolos requires a minimum conda version {CONDA_MIN_VERSION}, "
@@ -32,7 +36,7 @@ def verify_conda_version(output):
                 f"Please run 'conda update conda' or your preferred update method"
             )
     else:
-        clog.warn(
+        clog.warning(
             f"Could not determine conda version, resolos might not function correctly. "
             f"Please reinstall conda"
         )
@@ -73,8 +77,10 @@ def check_conda_env_exists_local(env_name: str):
 
 
 def create_conda_env_local(env_name: str):
+    # Python < 3.9 is required by conda-tree
+    # https://stackoverflow.com/questions/66174862/import-error-cant-import-name-gcd-from-fractions
     ret_val, output = run_shell_cmd(
-        f"conda create -y -n {env_name}",
+        f'conda create -y -n {env_name} "python<3.9"',
     )
     if ret_val != 0:
         raise ResolosException(
@@ -139,9 +145,9 @@ def create_conda_env_remote(remote_settings, env_name: str):
 def install_conda_remote(remote_settings):
     ret_val, output = run_ssh_cmd(
         remote_settings,
-        f"wget -q {CONDA_LINUX_INSTALLER_URL} -O ~/miniconda.sh "
-        f"&& bash ~/miniconda.sh -b -p ~/miniconda"
-        f"&& rm ~/miniconda.sh",
+        f"wget -q {CONDA_LINUX_INSTALLER_URL} -O {remote_settings['conda_install_path']}/miniconda.sh "
+        f"&& bash {remote_settings['conda_install_path']}/miniconda.sh -b -p {remote_settings['conda_install_path']}/miniconda"
+        f"&& rm {remote_settings['conda_install_path']}/miniconda.sh",
         stdout_as_info=True,
     )
     if ret_val != 0:
@@ -375,11 +381,11 @@ def get_requirements(env_name, filename=None):
         skip = False
         name = pkg.get("name")
         if name is None:
-            clog.warn(f"Could not get name of package {pkg}, will skip it")
+            clog.warning(f"Could not get name of package {pkg}, will skip it")
             skip = True
         version = pkg.get("version")
         if version is None:
-            clog.warn(f"Could not get version of package {pkg}, will skip it")
+            clog.warning(f"Could not get version of package {pkg}, will skip it")
             skip = True
         if not skip:
             res.append(f"{name}=={version}")
@@ -458,7 +464,6 @@ def sync_env_and_files(remote_settings):
         explicit_package_list(local_env, filename=explicit_packages_file)
         clog.info(f"Syncing project files...")
         sync_files(remote_settings)
-        clog.info(f"Successfully synced project files")
         if not check_conda_env_exists_remote(remote_settings, remote_env):
             create_conda_env_remote(remote_settings, remote_env)
         try:
@@ -466,6 +471,9 @@ def sync_env_and_files(remote_settings):
                 f"install --name {remote_env} --file {remote_path}/.env/spec-file.txt",
                 remote_settings,
             )
+            project_remote_settings = read_project_remote_config(remote_id)
+            project_remote_settings["last_env_sync"] = datetime.utcnow()
+            write_project_remote_config(remote_id, project_remote_settings)
         except RemoteCommandError as ex:
             clog.info(
                 f"Failed to sync conda env to remote '{remote_id}' using explicit packages list, "
@@ -475,21 +483,24 @@ def sync_env_and_files(remote_settings):
             pack_conda_env(local_env, pack_file)
             sync_files(remote_settings)
             remote_env_name = f"resolos_env_{randomString()}"
-            run_ssh_cmd(
+            ret_val, output = run_ssh_cmd(
                 remote_settings,
                 f"mkdir -p ./.resolos/envs/{remote_env_name} && "
                 f"tar -xzf {remote_path}/.env/conda_pack.tar.gz -C ./.resolos/envs/{remote_env_name} && "
                 f"source ./.resolos/envs/{remote_env_name}/bin/activate && "
                 f"conda-unpack",
             )
-            prdc = get_project_remote_dict_config()
-            project_remote_config = prdc.read()
-            if remote_id not in project_remote_config:
-                project_remote_config[remote_id] = {}
-            project_remote_config[remote_id][
+            if ret_val != 0:
+                raise RemoteCommandError(
+                    f"Could not use conda-pack to sync the environment to the remote, "
+                    f"the error message was:\n{output}\n"
+                )
+            project_remote_settings = read_project_remote_config(remote_id)
+            project_remote_settings[
                 "env_name"
             ] = f"source ./.resolos/envs/{remote_env_name}/bin/activate"
-            prdc.write(project_remote_config)
+            project_remote_settings["last_env_sync"] = datetime.utcnow()
+            write_project_remote_config(remote_id, project_remote_settings)
     else:
         clog.info(
             f"The local machine has a different platform or OS as the remote (linux, x86_64), will use the environment "
@@ -501,12 +512,14 @@ def sync_env_and_files(remote_settings):
         export_conda_env(local_env, filename=env_history_file)
         clog.info("Syncing project files...")
         sync_files(remote_settings)
-        clog.info(f"Successfully synced project files")
         try:
             execute_remote_conda_command(
                 f"env update -n {remote_env} -f {remote_path}/.env/env.yaml",
                 remote_settings,
             )
+            project_remote_settings = read_project_remote_config(remote_id)
+            project_remote_settings["last_env_sync"] = datetime.utcnow()
+            write_project_remote_config(remote_id, project_remote_settings)
         except RemoteCommandError as ex:
             clog.info(
                 f"Failed to sync conda env to remote '{remote_id}' using complete environment description, "
@@ -517,6 +530,9 @@ def sync_env_and_files(remote_settings):
                     f"env update -n {remote_env} -f {remote_path}/.env/env_from_history.yaml",
                     remote_settings,
                 )
+                project_remote_settings = read_project_remote_config(remote_id)
+                project_remote_settings["last_env_sync"] = datetime.utcnow()
+                write_project_remote_config(remote_id, project_remote_settings)
             except RemoteCommandError:
                 clog.info(
                     f"Failed to sync conda env to remote '{remote_id}' using conda env --from-history, "
@@ -524,3 +540,6 @@ def sync_env_and_files(remote_settings):
                 )
                 nondep_packages = get_nondep_packages(local_env)
                 install_conda_packages(nondep_packages, target=remote_settings)
+                project_remote_settings = read_project_remote_config(remote_id)
+                project_remote_settings["last_env_sync"] = datetime.utcnow()
+                write_project_remote_config(remote_id, project_remote_settings)
