@@ -1,25 +1,22 @@
 import click
-from .platform import find_project_dir
 from .config import (
     initialize_user_configs,
-    get_default_config_path,
-    randomString,
-    get_project_dict_config,
-    get_project_remote_dict_config,
     get_project_env,
     get_project_settings_for_remote,
     info,
+    verify_mutually_exclusive_options,
 )
-import click_logging
+import click_log
 from .logging import clog
 from .remote import (
     get_remote,
-    add_remote,
-    delete_remote,
     list_remote_ids,
     read_remote_db,
-    update_remote_settings,
-    set_remote,
+)
+from .remote_configuration import (
+    add_remote_configuration,
+    update_remote_configuration,
+    teardown_remote_configuration,
 )
 from .check import check_target, check, setup_ssh
 from .unison import sync_files
@@ -28,18 +25,16 @@ from .conda import (
     install_conda_packages,
     uninstall_conda_packages,
     sync_env_and_files,
-    check_conda_env_exists_remote,
-    create_conda_env_remote,
 )
 from .archive import make_archive, load_archive
 from .job import job_cancel, job_list, job_status, job_submit, job_run
-from .exception import NoRemotesError, NotAProjectFolderError
-from .init import init_project
+from .exception import NoRemotesError
+from .init import init_project, teardown
 import yaml
 
 
 @click.group("res")
-@click_logging.simple_verbosity_option(clog)
+@click_log.simple_verbosity_option(clog)
 @click.pass_context
 def res(ctx):
     """
@@ -70,10 +65,48 @@ def res(ctx):
     help="The path of the project folder on the remote. Will be created if not exists",
 )
 @click.option(
-    "-s",
-    "--source",
-    help="The source archive to initialize the project from. "
-    "Can be a path on the filesystem, or a download URL of the archive file",
+    "--base-url",
+    type=str,
+    envvar="YARETA_BASE_URL",
+    default="https://access.yareta.unige.ch",
+    help="The base url of the Yareta API",
+)
+@click.option(
+    "-a",
+    "--access-token",
+    type=str,
+    envvar="YARETA_ACCESS_TOKEN",
+    help="The personal DLCM access token",
+)
+@click.option(
+    "-d",
+    "--deposit-id",
+    type=str,
+    help="The deposit id of the Yareta deposit",
+)
+@click.option(
+    "-f",
+    "--filename",
+    type=click.Path(),
+    help="The filename of the archive to load",
+)
+@click.option(
+    "-u",
+    "--url",
+    type=str,
+    help="The publicly accessible url to load the archive from",
+)
+@click.option(
+    "-y",
+    is_flag=True,
+    help="If specified, no prompts will be displayed to install conda/unison.",
+    required=False,
+)
+@click.option(
+    "--no-remote-setup",
+    is_flag=True,
+    help="If specified, remote configuration will be skipped (syncing of project files and environment)",
+    required=False,
 )
 @click.pass_context
 def res_init(ctx, **kwargs):
@@ -86,16 +119,60 @@ def res_init(ctx, **kwargs):
 
     """
 
-    check_target()
+    verify_mutually_exclusive_options(
+        ["url", "filename", "deposit_id"],
+        ["--url", "--filename", "--deposit-id"],
+        must_select_one=False,
+        **kwargs,
+    )
+    check_target(no_confirm=kwargs.get("y", False))
     init_project(
-        kwargs.get("source"),
+        base_url=kwargs.get("base_url"),
+        access_token=kwargs.get("access_token"),
+        url=kwargs.get("url"),
+        filename=kwargs.get("filename"),
+        deposit_id=kwargs.get("deposit_id"),
         local_env_name=kwargs.get("env_name"),
         remote_env_name=kwargs.get("remote_env_name"),
         remote_files_path=kwargs.get("remote_path"),
+        no_confirm=kwargs.get("y", False),
+        no_to_remote_setup=kwargs.get("no_remote_setup"),
     )
 
 
+@res.command("teardown")
+@click.option(
+    "--skip-local",
+    is_flag=True,
+    help="If specified, the local environment and the .resolos folder will not be deleted",
+    required=False,
+)
+@click.option(
+    "--skip-remotes",
+    is_flag=True,
+    help="If specified, the remote environment and the synced files will not be deleted",
+    required=False,
+)
+@click.pass_context
+def res_teardown(ctx, **kwargs):
+    """
+    Reverses the steps of resolos init:
+
+    - Clears the project configuration folder .resolos
+
+    - Removes the linked conda environment
+
+    """
+    teardown(kwargs.get("skip_local", False), kwargs.get("skip_remotes", False))
+
+
 @res.command("check")
+@click.option(
+    "--raise-on-error",
+    is_flag=True,
+    help="Do not offer installing missing dependencies on remotes, raise exception instead",
+    required=False,
+)
 @click.pass_context
 def res_check(ctx, **kwargs):
     """
@@ -103,7 +180,7 @@ def res_check(ctx, **kwargs):
     Must be called from a resolos project
 
     """
-    check()
+    check(kwargs.get("raise_on_error", False))
 
 
 @res.command("info")
@@ -125,7 +202,7 @@ def res_info(ctx, **kwargs):
     required=False,
 )
 @click.pass_context
-def res_init(ctx, **kwargs):
+def res_setup_ssh(ctx, **kwargs):
     """
     Configures passwordless SSH access via SSH keys.
     Only needs to be run once per remote.
@@ -176,6 +253,12 @@ def res_remote(ctx):
     default="source ~/miniconda/bin/activate",
     help="The command that makes the 'conda' command available for the shell",
 )
+@click.option(
+    "--conda-install-path",
+    type=str,
+    default="~",
+    help="The path to install miniconda on the remote, if required",
+)
 @click.option(
     "--unison-path",
     type=str,
@@ -193,62 +276,24 @@ def res_remote(ctx):
     type=str,
     help="The path of the project folder on the remote. Will be created if not exists",
 )
+@click.option(
+    "--no-remote-setup",
+    is_flag=True,
+    help="If specified, remote configuration will be skipped (ssh key setup, syncing of project files and environment)",
+    required=False,
+)
+@click.option(
+    "-y",
+    is_flag=True,
+    help="If specified, SSH key setup, conda and unison install will happen without confirmation.",
+    required=False,
+)
 @click.pass_context
 def res_remote_add(ctx, **kwargs):
     """
     Adds a new remote with name 'name' to the Resolos configuration
     """
-    # print(ctx.obj)
-    remote_name = kwargs.get("name")
-    create_dict = {
-        "username": kwargs.get("username"),
-        "hostname": kwargs.get("hostname"),
-        "port": kwargs.get("port"),
-        "scheduler": kwargs.get("scheduler"),
-        "conda_load_command": kwargs.get("conda_load_command"),
-        "unison_path": kwargs.get("unison_path"),
-    }
-    add_remote(read_remote_db(), remote_name, create_dict)
-    remote_settings = get_remote(read_remote_db(), remote_name)
-    if click.confirm(
-        f"Do you want resolos to use its own SSH key for accessing the remote?",
-        default=True,
-    ):
-        setup_ssh(remote_settings)
-    clog.info(f"Running checks on new remote '{remote_name}'...")
-    check_target(remote_settings)
-    try:
-        project_dir = find_project_dir()
-        project_remote_config = get_project_remote_dict_config().read()
-        files_path = (
-            kwargs.get("remote_path")
-            or f"./resolos_projects/{project_dir.name}_{randomString()}"
-        )
-        remote_env_name = (
-            kwargs.get("remote_env_name") or f"resolos_env_{randomString()}"
-        )
-        project_remote_config[remote_name] = {
-            "env_name": remote_env_name,
-            "files_path": files_path,
-        }
-        get_project_remote_dict_config().write(project_remote_config)
-        if not check_conda_env_exists_remote(create_dict, remote_env_name):
-            if click.confirm(
-                f"Remote conda environment '{remote_env_name}' does not exists yet. "
-                f"Do you want to create it now?",
-                default=True,
-            ):
-                create_conda_env_remote(create_dict, remote_env_name)
-        else:
-            clog.info(
-                f"Remote conda environment '{remote_env_name}' already exists, continuing..."
-            )
-    except NotAProjectFolderError:
-        clog.debug(
-            f"Command was not executed from a project folder, no project set up was done"
-        )
-    clog.info(f"Remote {remote_name} added!")
-    return
+    add_remote_configuration(**kwargs)
 
 
 @res_remote.command("update")
@@ -276,6 +321,12 @@ def res_remote_add(ctx, **kwargs):
     type=str,
     help="The command that makes the 'conda' command available for the shell",
 )
+@click.option(
+    "--conda-install-path",
+    type=str,
+    default="~",
+    help="The path to install miniconda on the remote, if required",
+)
 @click.option(
     "--unison-path",
     type=str,
@@ -292,72 +343,34 @@ def res_remote_add(ctx, **kwargs):
     type=str,
     help="The path of the project folder on the remote. Will be created if not exists",
 )
+@click.option(
+    "-y",
+    is_flag=True,
+    help="If specified, SSH key setup, conda and unison install will happen without confirmation.",
+    required=False,
+)
 @click.pass_context
-def res_remote_add(ctx, **kwargs):
+def res_remote_update(ctx, **kwargs):
     """
     Updates existing remote with name 'name' in the Resolos configuration
     """
-    remote_id = kwargs["name"]
-    db = read_remote_db()
-    update_dict = update_remote_settings(db, remote_id, **kwargs)
-    update_dict["name"] = remote_id
-    clog.debug(f"The new remote config is:\n\n{update_dict}")
-    clog.info(f"Running checks on remote '{remote_id}'...")
-    check_target(update_dict)
-    try:
-        project_dir = find_project_dir()
-        project_remote_config = get_project_remote_dict_config().read()
-        project_remote_settings = project_remote_config.get(remote_id)
-        if project_remote_settings is None:
-            files_path = (
-                kwargs.get("remote_path")
-                or f"./resolos_projects/{project_dir.name}_{randomString()}"
-            )
-            remote_env_name = (
-                kwargs.get("remote_env_name") or f"resolos_env_{randomString()}"
-            )
-            project_remote_config[remote_id] = {
-                "env_name": remote_env_name,
-                "files_path": files_path,
-            }
-        else:
-            if kwargs.get("remote_env_name"):
-                remote_env_name = kwargs.get("remote_env_name")
-                project_remote_settings["env_name"] = remote_env_name
-            else:
-                remote_env_name = project_remote_settings.get("env_name")
-            if kwargs.get("remote_path"):
-                project_remote_settings["files_path"] = kwargs.get("remote_path")
-        get_project_remote_dict_config().write(project_remote_config)
-        if not check_conda_env_exists_remote(update_dict, remote_env_name):
-            if click.confirm(
-                f"Remote conda environment '{remote_env_name}' does not exists yet. "
-                f"Do you want to create it now?",
-                default=True,
-            ):
-                create_conda_env_remote(update_dict, remote_env_name)
-        else:
-            clog.info(
-                f"Remote conda environment '{remote_env_name}' already exists, continuing..."
-            )
-    except NotAProjectFolderError:
-        clog.debug(
-            f"Command was not executed from a project folder, no project set up was done"
-        )
-    set_remote(db, remote_id, update_dict)
-    clog.info(f"Remote {remote_id} successfully  modified!")
-    return
+    update_remote_configuration(**kwargs)
 
 
 @res_remote.command("remove")
 @click.argument("name")
+@click.option(
+    "--purge",
+    is_flag=True,
+    help="Removes all synced project files and environments from the remote",
+    required=False,
+)
 @click.pass_context
 def res_remote_remove(ctx, **kwargs):
     """
     Removes the remote from the Resolos configuration
     """
-    delete_remote(read_remote_db(), kwargs.get("name"))
-    clog.info(f"Removed remote {kwargs.get('name')}!")
+    teardown_remote_configuration(**kwargs)
 
 
 @res_remote.command("list")
@@ -401,7 +414,7 @@ def res_sync(ctx, **kwargs):
 @res.command("run")
 @click.argument("command", type=str)
 @click.pass_context
-def res_job(ctx, **kwargs):
+def res_run(ctx, **kwargs):
     cmd = kwargs.get("command")
     local_env = get_project_env()
     execute_command_in_local_conda_env(cmd, local_env)
@@ -519,30 +532,133 @@ def res_archive(ctx):
 
 
 @res_archive.command("create")
-@click.argument("output", type=click.Path())
+@click.option(
+    "--base-url",
+    type=str,
+    envvar="YARETA_BASE_URL",
+    default="https://access.yareta.unige.ch",
+    help="The base url of the Yareta API",
+)
+@click.option(
+    "-a",
+    "--access-token",
+    type=str,
+    envvar="YARETA_ACCESS_TOKEN",
+    help="The personal DLCM access token",
+)
+@click.option(
+    "-o",
+    "--organizational-unit-id",
+    type=str,
+    envvar="YARETA_ORG_UNIT_ID",
+    help="The resource id of the organizational unit",
+)
+@click.option(
+    "-f",
+    "--filename",
+    type=click.Path(),
+    default=None,
+    help="The filename of the archive",
+)
+@click.option(
+    "-t",
+    "--title",
+    type=str,
+    help="The title of the Yareta deposit to create for the archive",
+)
+@click.option(
+    "-y",
+    "--year",
+    type=str,
+    help="The year of the Yareta deposit",
+)
+@click.option(
+    "-desc",
+    "--description",
+    type=str,
+    help="The description of the Yareta deposit",
+)
+@click.option(
+    "--deposit-access",
+    type=str,
+    default="PUBLIC",
+    help="The access level of the Yareta deposit",
+)
+@click.option(
+    "--license-id",
+    type=str,
+    default="CC-BY-4-0",
+    help="The license id of the Yareta deposit",
+)
+@click.option(
+    "--keywords",
+    type=str,
+    help="Comma-separated list of the Yareta deposit keywords",
+)
 @click.pass_context
-def res_archive_create(ctx, output, **kwargs):
+def res_archive_create(ctx, **kwargs):
     """
-    Archives the project to the specified destination.
+    Archives the project to the specified destination. The currently supported destinations are local file (-f, --filename)
+    and Yareta archive (-o, --organizational-unit-id).
 
-    Output must be a filesystem path (e.g. ../res_v1.tar.gz) writeable for the the resolos process.
-    The path should not be inside the project folder.
+    Notes for the local file destination:
+
+        --filename:
+        Filename must be a filesystem path (e.g. ../res_v1.tar.gz) writeable for the the resolos process.
+        The path should not be inside the project folder.
     """
     local_env = get_project_env()
-    make_archive(local_env, output)
+    make_archive(local_env, **kwargs)
 
 
 @res_archive.command("load")
-@click.argument("source")
+@click.option(
+    "--base-url",
+    type=str,
+    envvar="YARETA_BASE_URL",
+    default="https://access.yareta.unige.ch",
+    help="The base url of the Yareta API",
+)
+@click.option(
+    "-a",
+    "--access-token",
+    type=str,
+    envvar="YARETA_ACCESS_TOKEN",
+    help="The personal DLCM access token",
+)
+@click.option(
+    "-d",
+    "--deposit-id",
+    type=str,
+    help="The deposit id of the Yareta deposit",
+)
+@click.option(
+    "-f",
+    "--filename",
+    type=click.Path(),
+    help="The filename of the archive to load",
+)
+@click.option(
+    "-u",
+    "--url",
+    type=str,
+    help="The url to load the archive from",
+)
+@click.option(
+    "-y",
+    is_flag=True,
+    help="If specified, the archive will be loaded without a confirmation prompt.",
+    required=False,
+)
 @click.pass_context
-def res_archive_load(ctx, source, **kwargs):
+def res_archive_load(ctx, **kwargs):
     """
     Loads the specified archive into the project.
 
     Source can be a filesystem path or a publicly accessible https download url.
     """
 
-    load_archive(source)
+    load_archive(confirm_needed=not kwargs.get("y"), **kwargs)
 
 
 @res.command("install")
